import logging
logging.basicConfig(level=logging.ERROR)

import nose
import numpy as np
from minpower.tests.test_utils import istest
from minpower.tests.test_integration import run_case

mipgap = 0.0001


@istest
def run_basic_checks():
    '''
    solutions should all make sense (cost=0 and power=0 if status=0)
    '''
    run_case('stochastic_short_case', deterministic_solve=True)


@istest
def mock_stochastic():
    '''
    the solution to the stochastic problem with only two scenarios,
    both with values equal to the deterministic forecast and prob=0.5,
    should have a total cost equal (within solver tolerance) to
    the deterministic forecat solution total cost
    '''
    slnS = run_case('stochastic_mock_case',
        scenarios_directory='scenarios/')
    slnD = run_case('stochastic_mock_case',
        deterministic_solve=True)

    costS = slnS.observed_cost.sum().sum()
    costD = slnD.observed_cost.sum().sum()

    diffpercent = np.abs((costS - costD) / costD)
    assert(diffpercent < mipgap)


@istest
def standalone():
    try:
        import tables
    except ImportError:
        raise nose.SkipTest('standalone mode requires pytables')

    slnA = run_case('stochastic_short_case', deterministic_solve=True)
    slnB = run_case('stochastic_short_case', deterministic_solve=True, standalone=True)

    assert(slnA.observed_cost.sum().sum() == slnB.observed_cost.sum().sum())



"""
Stochastic scenario models for schedules. 
"""
from coopr.pyomo import AbstractModel, Set, Param, Boolean, Var
from coopr.pysp.scenariotree import ScenarioTree
from coopr.pysp.ef import create_ef_instance
import gc, logging

def construct_simple_scenario_tree(probabilities, time_stage=None):
    '''Construct a simple scenario tree instance'''
    tree=new_scenario_tree_model()

    prob_set = range(len(probabilities))
    if time_stage is None:
        scenario_names=['s{n}'.format(n=n) for n in prob_set]
        node_names=['n{n}'.format(n=n) for n in prob_set]
    else:
        scenario_names=['s{n}t{t}'.format(n=n, t=time_stage) for n in prob_set]
        node_names=['n{n}t{t}'.format(n=n, t=time_stage) for n in prob_set]
        

    tree.Stages.add('first stage','second stage')
    tree.Nodes.add('root')
    for nm in node_names: tree.Nodes.add(nm)
    tree.NodeStage['root']='first stage'
    for nm in node_names: tree.NodeStage[nm]='second stage'
    tree.Children['root']=node_names
    
    tree.ConditionalProbability['root']=1.0
    for i,pr in enumerate(probabilities): tree.ConditionalProbability[node_names[i]]=pr

    for nm in scenario_names: tree.Scenarios.add(nm)        
    for i,nm in enumerate(scenario_names): tree.ScenarioLeafNode[nm]=node_names[i]
    # tree.Scenarios.pprint()

    # note that stage variables still need definition
    # as do the values of the per scenario variables
    return tree

def define_stage_variables(scenario_tree, power_system, times):
    # scenario_tree.Stages.pprint()
    # scenario_tree.StageVariables.pprint()

    #create sets of variable names (not actual variables) for each stage
    variables_first_stage = Set()
    variables_second_stage = Set()

    for gen in power_system.get_generators_without_scenarios():
        # for each non stochastic generator
        if gen.is_controllable: 
            variables_first_stage.add(
                str(gen.get_variable('status',indexed=True,time=None))+'[*]')
            variables_second_stage.add(
                str(gen.get_variable('power',indexed=True,time=None))+'[*]')
        # note - appending '[*]' to the indicies is required to get 
        # pysp to assign all the variables in the array to a shape
        
    # variables_first_stage.pprint()
    scenario_tree.StageVariables['first stage'] = variables_first_stage
    scenario_tree.StageVariables['second stage'] = variables_second_stage
    
    #power_system._model=power_system._model.create()
    #power_system._model.pprint()
    scenario_tree.StageCostVariable['first stage'] = str(power_system.cost_first_stage())
    scenario_tree.StageCostVariable['second stage'] = str(power_system.cost_second_stage())

def create_problem_with_scenarios(power_system,times,scenariotreeinstance,stage_hours,overlap_hours, stage_number):
        
    #for node in scenariotreeinstance.nodes:
    
    scenario_tree = ScenarioTree(
        scenarioinstance=power_system._model, 
        scenariotreeinstance=scenariotreeinstance)


    if not scenario_tree.validate(): 
        for s,scenario in enumerate(scenario_tree._scenarios):
            print s,scenario
        raise ValueError('not a valid scenario tree')

    gen = power_system.get_generator_with_scenarios()
    
    scenario_instances = {}

    #construct scenario instances
    gc.disable()  
    for s,scenario in enumerate(scenario_tree._scenarios):
        scenario_instance = power_system._model.clone()
        
        power=getattr(scenario_instance,'power_{}'.format(str(gen)))
        #set the values of the parameter for this scenario
        logging.debug('setting scenario values for s%i'%s)
        scenario_vals = gen._get_scenario_values(times, s=s)        
        for t, time in enumerate(times): power[time] = scenario_vals[t]
        
        #power.pprint()
        scenario_instance.preprocess()
        scenario_instances[scenario._name] = scenario_instance    

    gc.enable()
    scenario_tree.defineVariableIndexSets(scenario_instances)
    full_problem_instance = create_ef_instance(scenario_tree, scenario_instances)
    # could generate cvar here
    #full_problem_instance.pprint()
    
    #relax the non-anticipatory constraints on the generator status variables beyond the UC time horizon
    for time in times.post_horizon():
        logging.debug('get rid of NA constraint at '+str(time))
        for scenario in scenario_instances.keys():
            for gen in power_system.generators():
                if not gen.is_controllable: continue
                u=gen.status().name
                #print 'clear constraint on {s}_{u}_{t}'.format(s=scenario,u=u,t=str(time))
                full_problem_instance._clear_attribute('{s}_{u}_{t}'.format(s=scenario,u=u,t=str(time)))
    
        
    power_system.stochastic_formulation = True
    power_system._stochastic_instance = full_problem_instance
    power_system._scenario_tree = scenario_tree
    power_system._scenario_instances = scenario_instances
    return power_system

def get_scenario_based_costs(scenario_tree,scenario_instances):
    #scenario_tree.pprintCosts(scenario_instances)
    costs=dict()
    for node in scenario_tree._tree_node_map.values():
        scenarios=node._scenarios
        if len(scenarios)==1: 
            costs[node._scenarios[0]._name] = \
                node.computeExpectedNodeCost(scenario_instances)
        else: 
            continue
    return costs

def update_variables(power_system,times):
    '''Convert all variables into dictionaries of their solved values, keyed by scenario'''
    variable_names = power_system._model.active_components(Var).keys()
    values=dict([(nm, {}) for nm in variable_names])
    for s, scenario in power_system._scenario_instances.items():
        for var_name, var in scenario.active_components(Var).items():
            if var.is_indexed(): 
                    values[var_name][s] = dict(
                        [(idx, var_val.value) for idx, var_val in var.iteritems()])
            else: 
                    values[var_name][s] = var.value
    power_system._per_scenario_values = values


def new_scenario_tree_model():
    # https://software.sandia.gov/trac/coopr/browser/coopr.pysp/trunk/coopr/pysp/util/scenariomodels.py
    scenario_tree_model = AbstractModel()

    # all set/parameter values are strings, representing the names of various entities/variables.

    scenario_tree_model.Stages = Set(ordered=True)
    scenario_tree_model.Nodes = Set()

    scenario_tree_model.NodeStage = Param(scenario_tree_model.Nodes,
        within=scenario_tree_model.Stages)
    scenario_tree_model.Children = Set(scenario_tree_model.Nodes,
        within=scenario_tree_model.Nodes, ordered=True)
    scenario_tree_model.ConditionalProbability = Param(
        scenario_tree_model.Nodes)

    scenario_tree_model.Scenarios = Set(ordered=True)
    scenario_tree_model.ScenarioLeafNode = Param(scenario_tree_model.Scenarios,
        within=scenario_tree_model.Nodes)

    scenario_tree_model.StageVariables = Set(scenario_tree_model.Stages)
    scenario_tree_model.StageCostVariable = Param(scenario_tree_model.Stages)

    # scenario data can be populated in one of two ways. the first is "scenario-based",
    # in which a single .dat file contains all of the data for each scenario. the .dat
    # file prefix must correspond to the scenario name. the second is "node-based",
    # in which a single .dat file contains only the data for each node in the scenario
    # tree. the node-based method is more compact, but the scenario-based method is
    # often more natural when parameter data is generated via simulation. the default
    # is scenario-based.
    scenario_tree_model.ScenarioBasedData = Param(within=Boolean,
        default=True, mutable=True)

    # do we bundle, and if so, how?
    scenario_tree_model.Bundling = Param(within=Boolean,
        default=False, mutable=True)
    scenario_tree_model.Bundles = Set() # bundle names
    scenario_tree_model.BundleScenarios = Set(scenario_tree_model.Bundles)
    
    return scenario_tree_model



import sys
import clint
import subprocess
import os

def main():
    '''
    a wrapper to make the minpower script work with cluster schedulers
    or with an ssh call that you start on your laptop and then close the
    connection.
    '''
    minpower_args = ['minpower']+clint.args.all

    # subprocess style
    if not '-p' in minpower_args:
        minpower_args.append('-p')
    if not '--standalone' in minpower_args:
        minpower_args.append('--standalone')

    print minpower_args

    if 'hyak' not in os.uname()[1]:
        scheduler_args = ['nohup']
        stdout = open('{}.out'.format(os.getpid()),'w')
        stderr = open('{}.err'.format(os.getpid()),'w')
    else:
        scheduler_args = ['qsub',
            '-l nodes=1:ppn=12,feature=12core,mem=16gb,walltime=48:00:00',
            '-m ae', #mail on completion/failure
            '-M argreen@uw.edu'
            ]
        # see https://sig.washington.edu/itsigs/Hyak_Job_Scheduler
        # default walltime limit is only one hour - must set this

        # these get redirected by hyak anyway
        stdout = sys.stdout
        stderr = subprocess.STDOUT

        # check that paths are absolute
        case_dir = os.path.dirname(clint.args.files[0])
        if case_dir != os.path.abspath(case_dir):
            raise IOError('paths have to be absolute for hyak')

        # there should be a better way than having to call a script on disk
        script_name = './call{}.sh'.format(os.getpid())
        with open(script_name, 'w+') as f:
            f.write(' '.join(minpower_args))
        minpower_args = [script_name]

    pid = subprocess.Popen( scheduler_args + minpower_args,
        stdout=stdout,
        stderr=stderr,
        ).pid

    print 'parent process {}'.format(os.getpid())
    print 'starting run {}'.format(pid)


if __name__=='__main__': main()




import numpy as np
from commonscripts import update_attributes, pairwise
from optimization import value,OptimizationObject
from config import user_config
import re
from coopr.pyomo import Piecewise


class Bid(OptimizationObject):
    """
    A bid modeled by a polynomial or a set of piecewise points.
    """
    def __init__(self,
            polynomial='10P',
            bid_points=None,
            constant_term=0,
            owner=None,
            times=None,
            input_variable=0,
            min_input=0,
            max_input=1000,
            num_breakpoints=user_config.breakpoints,
            status_variable=True,
            fixed_input=False):
        update_attributes(self,locals(),exclude=['owner'])
        self._parent_problem=owner._parent_problem
        self.owner_id=str(owner)
        
        self.is_pwl = (self.bid_points is not None)
        self.is_linear = is_linear(self.polynomial)
        
        if not fixed_input: self.build_model()
    def build_model(self): 
        if self.bid_points is None:
            
            self.is_pwl = False
            self.constant_term = self.polynomial[0]
            
            if self.is_linear: return
            
            polynomial = list(self.polynomial)
            # use constant term in place of the 0th order term
            polynomial[0] = 0 
            
            self.add_variable('cost', index=self.times.set, low=0)
            def pw_rule(model,time,input_var): 
                return polynomial_value(polynomial, input_var)
            self.discrete_input_points=discretize_range(self.num_breakpoints,self.min_input,self.max_input)
            in_pts=dict((t,self.discrete_input_points) for t in self.times.set)

            pw_representation = Piecewise(
                self.times.set,
                self.get_variable('cost', time=None, indexed=True),
                self.input_variable(),
                f_rule=pw_rule,
                pw_pts=in_pts,
                pw_constr_type='LB',
                warn_domain_coverage = False, 
                # unless warn_domain_coverage is set, pyomo will complain
                # gen lower power bounds are set to zero (status trick) 
                # and Piecewise complains if Pmin>0,
               )
            
        else:
            # custom bid points
            self.is_linear = False
            self.is_pwl = True
            self.add_variable('cost', index=self.times.set, low=0)            
            self.discrete_input_points = self.bid_points.power.values.tolist()
            in_pts = dict((t, self.discrete_input_points) for t in self.times.set)
            mapping = self.bid_points.set_index('power').to_dict()['cost']
            def pw_rule_points(model, time, input_var): 
                # just the input->output points mapping in this case
                # see coopr/examples/pyomo/piecewise/example3.py
                return mapping[input_var]
                
            pw_representation = Piecewise(self.times.set,
                self.get_variable('cost',time=None,indexed=True),
                self.input_variable(),
                pw_pts=in_pts,
                pw_constr_type='LB',
                pw_repn='DCC', # the disagregated convex combination method 
                f_rule=pw_rule_points,
                warn_domain_coverage=False)

        pw_representation.name = self.iden()
        self.max_output = pw_representation._f_rule(None, None, self.max_input)
        self._parent_problem().add_component_to_problem(pw_representation)
            
                
            
    def output(self, time=None, scenario=None, evaluate=False):
        status = self.status_variable(time, scenario)
        power = self.input_variable(time, scenario)        
        if evaluate: 
            status = value(status)
            power = value(power)

        if self.is_linear: 
            out = self.polynomial[1]*power
        else: 
            out = self.get_variable('cost', 
                time=time, scenario=scenario, indexed=True)
            if evaluate: out = value(out)
        
        if self.constant_term != 0:
            out += status * self.constant_term
            
        return out

    def output_true(self, input_var, force_linear=False): 
        '''true output value of bid'''
        input_val = value(input_var)
        
        if (self.is_pwl or force_linear) and not self.is_linear:
            if not self.is_pwl and self.bid_points is None:
                # construct the bid points
                bid_pt_outputs = map(lambda pt: polynomial_value(self.polynomial, pt), self.discrete_input_points)
                self.bid_points = zip(self.discrete_input_points, bid_pt_outputs)
            
            
            for A, B in pairwise(self.bid_points.values.tolist()):
                if A[0] <= input_val <= B[0]: 
                    return get_line_value(A, B, input_val) + self.constant_term
        else: 
            return polynomial_value(self.polynomial, input_val)

    def output_incremental(self,input_var):
        input_val = value(input_var)
        if self.is_pwl:
            for A,B in pairwise(self.bid_points):
                if A[0]<=input_val<=B[0]: return get_line_slope(A,B)
        else:
            return polynomial_incremental_value(self.polynomial,value(input_var))

    def output_incremental_range(self):
        if self.is_pwl: 
            input_range = self.discrete_input_points
            output_range = [0]
            for A,B in pairwise(self.bid_points):
                output_range.append( get_line_slope(A,B) )
        else:
            input_range = np.arange(self.min_input, self.max_input, 1.0)
            output_range = [polynomial_incremental_value(self.polynomial,x) 
                for x in input_range]
        return input_range,output_range
    def __str__(self): return 'bid_{}'.format(self.owner_id)
    def iden(self,*a,**k): return 'bid_{}'.format(self.owner_id)

def is_linear(coefs):
    result = False
    if coefs is None: result = False
    else:
        if len(coefs) < 2: result = True
        elif all(m==0 for m in coefs[2:]): result = True
        else: result = False
    return result

def discretize_range(num_breakpoints,minimum,maximum):
    step = (maximum-minimum)/float(num_breakpoints-1)
    return [x * step + minimum for x in range(int(num_breakpoints))]

def polynomial_value(multipliers,variable):
    """get the value of a polynomial"""
    def term(mult,var,order):
        if order>1: return mult*variable**order
        elif order==1: return mult*variable
        elif order==0: return mult
    return sum([term(mult,variable,order) for order,mult in enumerate(multipliers)])

def polynomial_incremental_value(multipliers,variable):
    """get the incremental value of a polynomial"""
    return sum([(mult*order*variable**(order-1) if order>0 else 0) for order,mult in enumerate(multipliers)])

def parse_polynomial(s):
    """
    Parse a string into a set of multipliers.
    Heavily adapted from `<http://bit.ly/polynomialParse>`_.
    
    Can handle simple polynomials (addition and subtraction):     
    
    >>> parse_polynomial('7x^2 + 6x - 5')
    [-5, 6, 7]
    
    or with the explicit * multiplier:
    
    >>> parse_polynomial('7*P^2 + 6*P - 5')
    [-5, 6, 7]
    
    or even with the terms in some random order: 
    
    >>> parse_polynomial('6*P - 5 + 7*P^2')
    [-5, 6, 7]
    """
    
    def parse_n(s):
        '''Parse the number part of a polynomial string term'''
        if not s: return 1
        elif s == '-': return -1
        elif s == '+': return 1
        return eval(s)

    def parse_p(s,powerPattern):
        '''Parse the power part of a polynomial string term'''
        if not s: return 0
        multipliers = powerPattern.findall(s)[0]
        if not multipliers: return 1
        return int(multipliers)
    s=str(s).replace(' ','') #remove all whitespace from string
    m = re.search('[a-zA-Z]+', s) 
    try: varLetter=m.group(0)
    except AttributeError: varLetter='P'
    termPattern = re.compile('([+-]?\d*\.?\d*)\**({var}?\^?\d?)'.format(var=varLetter))
    powerPattern = re.compile('{var}\^?(\d)?'.format(var=varLetter))
    order_multipliers = {}
    
    for n,p in termPattern.findall(s):
        n,p = n.strip(),p.strip()
        if not n and not p: continue
        n,p = parse_n(n),parse_p(p,powerPattern)
        if order_multipliers.has_key(p): order_multipliers[p] += n
        else: order_multipliers[p] = n
    highest_order = max(max(order_multipliers.keys()),1) #order must be at least linear
    multipliers = [0]*(highest_order+1)
    for key,value in order_multipliers.items(): multipliers[key] = value
    
    return multipliers

def get_line_slope(A,B):
    xA,yA = A
    xB,yB = B
    return (yB-yA)*1.0/(xB-xA)


def get_line_value(A,B,x):
    '''
    take a pair of points and make a linear function
    get the value of the function at x
    see http://bit.ly/Pd4z4l
    '''
    xA,yA = A
    slope = get_line_slope(A,B)
    return slope*(value(x) - xA) + yA



"""
Power systems optimization problem solver
for ED, OPF, UC, and SCUC problems. The :mod:`solve`
module contains the top-level commands for creating
problems and solving them.
"""

import sys, os, logging, subprocess
import time as timer
import argparse
import pdb
from optimization import OptimizationError
import get_data, powersystems, stochastic, results
from config import user_config
from commonscripts import joindir, StreamToLogger, set_trace
from standalone import store_times, init_store, get_storage, repack_storage

def _set_store_filename(pid=None):
    fnm = 'stage-store.hd5'
    if user_config.output_prefix:
        fnm = '{}-{}'.format(pid, fnm)

    user_config.store_filename = joindir(user_config.directory, fnm)

def solve_multistage_standalone(power_system, times, scenario_tree, data):

    stage_times = times.subdivide(
        user_config.hours_commitment, user_config.hours_overlap)

    storage = init_store(power_system, stage_times, data)

    for stg,t_stage in enumerate(stage_times):
        logging.info('Stage starting at {}'.format(t_stage.Start.date()))
        # store current stage times
        storage = store_times(t_stage, storage)
        storage.close()
        storage = None
        command = 'standalone_minpower {dir} {stg} {pid}'.format(
                dir=user_config.directory, stg=stg,
                pid='--pid {}'.format(os.getpid()) if user_config.output_prefix else '')
        try: subprocess.check_call(command, shell=True, stdout=sys.stdout)
        except AttributeError:
            # HACk - avoid error when nose hijacks sys.stdout
            subprocess.check_call(command, shell=True)

    repack_storage()
    storage = get_storage()
    return storage, stage_times

def standaloneUC():
    '''the hook for the ``standalone_minpower`` script'''
    from standalone import store_state, load_state

    parser = argparse.ArgumentParser()
    parser.add_argument('directory', type=str, help='the problem direcory')
    parser.add_argument('stg', type=int, help='the stage number')
    parser.add_argument('--pid', type=int, default=None,
        help='the process id of the parent')

    args = parser.parse_args()
    stg = args.stg
    user_config.directory = args.directory
    if args.pid:
        user_config.output_prefix = True
    
    _set_store_filename(args.pid)

    _setup_logging(args.pid)

    # load stage data
    power_system, times, scenario_tree = load_state()

    sln = create_solve_problem(power_system, times, scenario_tree, stage_number=stg)

    store = store_state(power_system, times, sln)
    store.flush()

    return

def solve_problem(datadir='.',
        shell=True,
        problemfile=False,
        csv=True,
        ):
    """
    Solve a optimization problem specified by spreadsheets in a directory.
    Read in data, create and solve the problem, and return solution.
    The problem type is determined by the data.
    All options are set within `user_config`.
    """
    user_config.directory = datadir
    
    pid = os.getpid()
    _set_store_filename(pid)
    _setup_logging(pid)
    
    start_time = timer.time()
    logging.debug('Minpower reading {}'.format(datadir))
    generators, loads, lines, times, scenario_tree, data = get_data.parsedir()
    logging.debug('data read')
    power_system = powersystems.PowerSystem(generators,loads,lines)

    logging.debug('power system initialized')
    if times.spanhrs <= user_config.hours_commitment + user_config.hours_overlap:
        solution = create_solve_problem(power_system, times, scenario_tree)
    else: #split into multiple stages and solve
        if user_config.standalone:
            stage_solutions, stage_times = solve_multistage_standalone(
                power_system, times, scenario_tree, data)
        else:
            stage_solutions, stage_times = solve_multistage(
                power_system, times, scenario_tree, data)
        solution = results.make_multistage_solution(
            power_system, stage_times, stage_solutions)

    if shell:
        if user_config.output_prefix:
            stdout = sys.stdout
            sys.stdout = StreamToLogger()
            solution.show()
            sys.stdout = stdout
        solution.show()
    if csv and not user_config.standalone: solution.saveCSV()
    if user_config.visualization: solution.visualization()
    logging.info('total time: {}s'.format(timer.time()-start_time))

    if user_config.on_complete_script:
        os.system(user_config.on_complete_script)

    return solution


def solve_multistage(power_system, times, scenario_tree=None, data=None):
    stage_times = times.subdivide(
        user_config.hours_commitment, user_config.hours_overlap)

    stage_solutions = []

    for stg, t_stage in enumerate(stage_times):
        logging.info('Stage starting at {}'.format(t_stage.Start.date()))
        # solve
        solution = create_solve_problem(
            power_system, t_stage, scenario_tree, stg)
        # add to stage solutions
        stage_solutions.append(solution)
        # reset model
        power_system.reset_model()
        # set inital state for next stage
        if stg < len(stage_times)-1:
            power_system.set_initialconditions(stage_times[stg+1].initialTime)

    return stage_solutions, stage_times


def create_solve_problem(power_system, times, scenario_tree=None,
    stage_number=None, rerun=False):
    '''create and solve an optimization problem.'''
    
    create_problem(power_system, times, scenario_tree,
        stage_number, rerun)
    
    instance = power_system.solve_problem(times)

    logging.debug('solved... get results')

    sln = results.make_solution(power_system, times)
    
    power_system.disallow_shedding()

    # resolve with observed power and fixed status
    if sln.is_stochastic:
        power_system.resolve_stochastic_with_observed(instance, sln)
    elif user_config.deterministic_solve or user_config.perfect_solve:
        power_system.resolve_determinisitc_with_observed(sln)
   
    if len(times)>1:
        power_system.get_finalconditions(sln)

        power_system.disallow_shedding()
        sln.stage_number = stage_number
    
    return sln

def create_problem(power_system, times, scenario_tree=None,
    stage_number=None, rerun=False):
    """Create an optimization problem."""

    logging.debug('initialized problem')
    power_system.create_variables(times)
    logging.debug('created variables')
    power_system.create_objective(times)
    logging.debug('created objective')
    power_system.create_constraints(times)
    logging.debug('created constraints')

    if scenario_tree is not None and sum(scenario_tree.shape)>0 and not rerun:
        gen = power_system.get_generator_with_scenarios()
        tree = stochastic.construct_simple_scenario_tree(
            gen.scenario_values[times.Start]['probability'].values.tolist(),
            time_stage=stage_number)

        logging.debug('constructed tree for stage %i'%stage_number)

        stochastic.define_stage_variables(tree, power_system, times)
        power_system = stochastic.create_problem_with_scenarios(
            power_system, times, tree,
            user_config.hours_commitment,
            user_config.hours_overlap,
            stage_number=stage_number)
    return

def _setup_logging(pid=None):
    ''' set up the logging to report on the status'''
    kwds = dict(
        level=user_config.logging_level,
        datefmt='%Y-%m-%d %H:%M:%S',
        format='%(asctime)s %(levelname)s: %(message)s')
    if user_config.output_prefix:
        kwds['filename'] = joindir(user_config.directory,
            '{}.pylog'.format(pid))
    if (user_config.logging_level > 10) and (not 'filename' in kwds):
        # don't log the time if debugging isn't turned on
        kwds['format'] = '%(levelname)s: %(message)s'
    logging.basicConfig(**kwds)


def main():
    '''
    The command line interface for minpower. For more info use:
    ``minpower --help``
    '''
    import os,traceback,sys

    parser = argparse.ArgumentParser(description='Minpower command line interface')
    parser.add_argument('directory', type=str,
        help='the direcory of the problem you want to solve')
    parser.add_argument('--solver','-s',  type=str,
        default=user_config.solver,
        help='the solver name (e.g. cplex, gurobi, glpk)')
    parser.add_argument('--visualization','-v',action="store_true",
        default=user_config.show_visualization,
        help='save a visualization of the solution')
    parser.add_argument('--breakpoints','-b',  type=int,
        default=user_config.breakpoints,
        help='number of breakpoints to use in piecewise linearization of polynomial costs')
    parser.add_argument('--hours_commitment','-c', type=int,
        default=user_config.hours_commitment,
        help='number hours per commitment in a rolling UC (exclusive of overlap)')
    parser.add_argument('--hours_overlap','-o', type=int,
        default=user_config.hours_overlap,
        help='number hours to overlap commitments in a rolling UC')

    parser.add_argument('--mipgap',  type=float,
        default=user_config.mipgap,
        help='the MIP gap solution tolerence')

    reserve = parser.add_argument_group('Reserve',
        'Does the system require reserve? The default is no reserve.')
    reserve.add_argument('--reserve_fixed', type=float,
        default=user_config.reserve_fixed,
        help='the static amount of reserve required at all times (in MW)')
    reserve.add_argument('--reserve_load_fraction', type=float,
        default=user_config.reserve_load_fraction,
        help='fraction of the total system load which is required as reserve')

    parser.add_argument('--duals','-d',action="store_true",
        default=user_config.duals,
        help='flag to get the duals, or prices, of the optimization problem')
    parser.add_argument('--dispatch_decommit_allowed', action="store_true",
        default=user_config.dispatch_decommit_allowed,
        help='flag to allow de-commitment of units in an ED -- useful for getting initial conditions for UCs')
    
    parser.add_argument('--cost_wind_shedding', type=float,
        default=user_config.cost_wind_shedding,
        help='the cost to the system to shed a MWh of wind energy')
    parser.add_argument('--cost_load_shedding', type=float,
        default=user_config.cost_load_shedding,
        help='the cost to the system to shed a MWh of load')
        
    
    stochastic = parser.add_argument_group('Stochastic UC',
        'options to modify the behavior of a stochastic problem')
    stochastic.add_argument('--scenarios',type=int,
        default=user_config.scenarios,
        help='limit the number of scenarios to N')        
    stochastic.add_argument('--faststart_resolve', '-F',
        action='store_true', default=user_config.faststart_resolve,
        help="""allow faststart units which are off to be
                started up during resolve with observed wind values""")

    stochastic_mode = stochastic.add_mutually_exclusive_group()
    stochastic_mode.add_argument('--deterministic_solve', '-D',
        action='store_true', default=user_config.deterministic_solve,
        help='solve a stochastic problem deterministically using the forecast_filename paramter')
    stochastic_mode.add_argument('--perfect_solve', '-P', action='store_true',
        default=user_config.perfect_solve,  #False
        help='solve a stochastic problem with perfect information')
    stochastic_mode.add_argument('--scenarios_directory', type=str,
        default=user_config.scenarios_directory,
        help='override scenarios directory for stochastic problem')
        

    parser.add_argument('--standalone', '-m', action="store_true", 
        default=user_config.standalone,
        help='Make each multi-day commitment its own subprocess (helps with memory issues).')
    parser.add_argument('--output_prefix','-p', action="store_true",
        default=user_config.output_prefix,
        help='Prefix all results files with the process id (for a record of simulataneous solves)')

    debugging = parser.add_argument_group('Debugging tools')
    debugging.add_argument('--debugger',action="store_true",
        default=user_config.debugger,
        help='use pdb when an error is raised')
    debugging.add_argument('--problem_file',action="store_true",
        default=user_config.problem_file,
        help='flag to write the problem formulation to a problem.lp file')        
    parser.add_argument('--logging_level', type=int,
        default=user_config.logging_level,
        help='set the level of detail for logging')
    debugging.add_argument('--profile',action="store_true",
        default=False,
        help='run cProfile and output to minpower.profile')
    
    ts = parser.add_argument_group('Timeseries modifiers',
        'Alter the timeseries after parsing the data.')
    ts.add_argument('--wind_multiplier', type=float,
        default=user_config.wind_multiplier,
        help='scale the wind power by this factor')
    ts.add_argument('--wind_forecast_adder', type=float,
        default=user_config.wind_forecast_adder,
        help='add a fixed amount (e.g. bias) to the wind power forecast')
    ts.add_argument('--load_multiplier', type=float,
        default=user_config.load_multiplier,
        help='scale the load power by this factor')
    
    
    constraints = parser.add_argument_group('Ignore/relax constraints',
        'Ignore or relax sets of constraints to allow for feasible solutions.')
    constraints.add_argument('--ignore_minhours_constraints', 
        action="store_true", default=user_config.ignore_minhours_constraints,
        help='drop the min up/down time constraints on the generators')
    constraints.add_argument('--ignore_ramping_constraints', 
        action="store_true", default=user_config.ignore_ramping_constraints,
        help='drop the min ramping power constraints on the generators')
    constraints.add_argument('--ignore_pmin_constraints', 
        action="store_true", default=user_config.ignore_pmin_constraints,
        help='drop the min power constraints on the generators')
    constraints.add_argument('--pmin_multiplier', type=float,
        default=user_config.pmin_multiplier,
        help='scale the generator minimum power limits by this factor')
    constraints.add_argument('--ramp_limit_multiplier', type=float,
        default=user_config.ramp_limit_multiplier,
        help='scale the generator ramp power limits by this factor')


    
    parser.add_argument('--on_complete_script', type=str, 
        default=user_config.on_complete_script,
        help='run a script on completion of the minpower script')
    # NOTE - don't let defaults creep into this defenition
    # that makes resetting the defaults during testing very hard 

    #figure out the command line arguments
    args = parser.parse_args()

    directory=args.directory

    if not os.path.isdir(directory):
        msg='There is no folder named "{}".'.format(directory)
        raise OSError(msg)

    user_config.update(vars(args))
    
    if args.profile:
        print 'run profile'
        import cProfile
        prof = cProfile.Profile()
        prof.runcall(solve_problem, directory)
        prof.dump_stats('minpower.profile')

    else:
        #solve the problem with those arguments
        try: solve_problem(directory)
        except:
            if args.debugger:
                __, __, tb = sys.exc_info()
                traceback.print_exc()
                pdb.post_mortem(tb)
            else: raise

# for use in dev
if __name__=='__main__': main()



from pandas import Series, DataFrame
from minpower import powersystems,schedule,solve
from minpower.generators import Generator
from minpower.config import user_config
from minpower.optimization import value, OptimizationError

import nose
from nose.tools import istest, with_setup, raises, set_trace
from nose.tools import eq_ as assert_equal


import logging
logging.basicConfig(
    level=logging.CRITICAL, 
    format='%(levelname)s: %(message)s')



default_config = user_config.copy()

def get_duals():
    user_config.duals = True
    
def reset_config():
    # note - can't set a new user_config, because it would be a local variable
    #   instead, we update it with the defaults
    #   this only works if we aren't adding anything extra to the user_config
    user_config.update(default_config)
    try: assert(user_config == default_config)
    except:
        print DataFrame(dict(
            config=Series(user_config), 
            default=Series(default_config) ))
        raise

singletime=schedule.just_one_time()


gen_costs = dict(cheap=10,mid=20,expensive=30)

def assertAlmostEqual(x,y):
    assert(round(x-y, 5)==0)


#def make_single_bus(generators,loads):
    #singlebus=powersystems.Bus()
    #singlebus.generators=generators
    #singlebus.loads=loads
    #return [singlebus]    

def make_cheap_gen(**kwargs):
    return Generator(name='cheap gen', costcurveequation='{}P'.format(gen_costs['cheap']), **kwargs)
def make_mid_gen(**kwargs):
    return Generator(name='middle-range gen', costcurveequation='{}P'.format(gen_costs['mid']), **kwargs)    
def make_expensive_gen(**kwargs):
    if 'costcurveequation' not in kwargs: kwargs['costcurveequation']='{}P'.format(gen_costs['expensive'])
    return Generator(name='expensive gen', **kwargs)    
def make_loads_times(Pd=200,Pdt=None,**kwargs):
    if Pdt is None:
        times = singletime
        sched = Series(Pd, index=times)
        loads=[powersystems.Load(schedule=sched, **kwargs)]
    else: 
        times = schedule.make_times_basic(N=len(Pdt))
        sched = Series(Pdt, index=times)
        loads=[powersystems.Load(schedule=sched,**kwargs)]
    
    return dict(loads=loads,times=times)

def solve_problem(generators, loads=None, times=None, 
    gen_init=None, lines=None, do_reset_config=True):
    
    if lines is None: lines=[]

    if len(times)>0: 
        for g,gen in enumerate(generators): 
            gen.index=g
            if gen_init is None: gen.set_initial_condition(times.initialTime)
            else:                gen.set_initial_condition(times.initialTime, **gen_init[g])
    
    power_system=powersystems.PowerSystem(generators,loads,lines)
    solve.create_problem(power_system,times)

    try: 
        instance = power_system.solve()
    except OptimizationError:
        #re-do stage, with load shedding allowed
        logging.critical('stage infeasible, re-running with load shedding.')
        power_system.allow_shedding(times)
        try:
            instance = power_system.solve()
        except OptimizationError:
            scheduled, committed = power_system.debug_infeasibe(times)
            set_trace()
            raise OptimizationError('failed to solve, even with load shedding.')
        

    if do_reset_config:
        reset_config()
    
    if power_system.solved:
        power_system.update_variables()
    else:
        #logging.critical( [g.power[times.initialTime] for g in generators] )
        power_system.write_model('problem.lp')
        raise OptimizationError('infeasible problem, wrote to problem.lp')
    return power_system,times



"""
Defines models for power systems components, including
:class:`~powersystems.PowerSystem`, :class:`~powersystems.Bus`,
:class:`~powersystems.Load` and  :class:`~powersystems.Line`.
:class:`~powersystems.Generator` components can be found
in the :module:`~generators`. Each of these objects inherits an
optimization framework from :class:`~optimization.OptimizationObject`.
"""

import logging
from coopr import pyomo
import numpy as np
import pandas as pd
from optimization import (value, dual, OptimizationObject, 
    OptimizationProblem, OptimizationError)
from commonscripts import (update_attributes, getattrL, flatten, set_trace)
from config import user_config

class Load(OptimizationObject):
    """
    Describes a power system load (demand).
    Currently only real power is considered.
    For OPF problems, the name of the bus can.
    For UC problems, schedules (pandas.Series objects) are used.
    By setting `sheddingallowed`, the amount of power can become a variable,
        (bounded to be at most the scheduled amount).
    """
    def __init__(self,
        name='', index=None, bus=None, schedule=None,
        sheddingallowed=True,
        cost_shedding=user_config.cost_load_shedding
        ):
        update_attributes(self,locals()) #load in inputs
        self.init_optimization()
        self.shedding_mode = False
        
    def power(self, time, evaluate=False):
        if self.shedding_mode:
            power = self.get_variable('power', time, indexed=True)
            if evaluate: power = value(power)
            return power
        else:
            return self.get_scheduled_output(time)
    def shed(self, time, evaluate=False): 
        return self.get_scheduled_output(time) - self.power(time,evaluate)
    def cost(self,time): return self.cost_shedding * self.shed(time)
    def cost_first_stage(self, times): return 0
    def cost_second_stage(self, times):
        return sum(self.cost(time) for time in times)
    def create_variables(self,times):
        if self.shedding_mode:
            self.add_variable('power', index=times.set, low=0)
    def create_constraints(self,times):
        if self.shedding_mode:
            for time in times:
                self.add_constraint('max_load_power',time,
                    self.power(time) <= self.get_scheduled_output(time))
    def create_objective(self,times):
        return sum([ self.cost(time) for time in times])

    def __str__(self): return 'd{ind}'.format(ind=self.index)
    def __int__(self): return self.index
    def iden(self, t): return str(self)+str(t)

    def get_scheduled_output(self, time):
        return float(self.schedule.ix[time])

class Line(OptimizationObject):
    """
    A tranmission line. Currently the model
    only considers real power flow under normal conditions.
    """
    def __init__(self, name='', index=None, frombus=None, tobus=None,
        reactance=0.05, pmin=None, pmax=9999, **kwargs):
        update_attributes(self,locals()) #load in inputs
        if self.pmin is None: 
            self.pmin=-1*self.pmax   # default is -1*pmax
        self.init_optimization()
    def power(self,time): return self.get_variable('power',time,indexed=True)
    def price(self,time):
        '''congestion price on line'''
        return dual(self.get_constraint('line flow',time))
    def create_variables(self,times):
        self.add_variable('power',index=times.set)
    def create_constraints(self,times,buses):
        '''create the constraints for a line over all times'''
        busNames=getattrL(buses,'name')
        iFrom,iTo=busNames.index(self.frombus),busNames.index(self.tobus)
        for t in times:
            line_flow_ij = self.power(t) == \
                1/self.reactance * (buses[iFrom].angle(t) - buses[iTo].angle(t))
            self.add_constraint('line flow',t,line_flow_ij)
            self.add_constraint('line limit high',t,self.power(t)<=self.pmax)
            self.add_constraint('line limit low',t,self.pmin<=self.power(t))
        return
    def __str__(self): return 'k{ind}'.format(ind=self.index)
    def __int__(self): return self.index
    def iden(self,t): return str(self)+str(t)


class Bus(OptimizationObject):
    """
    A transmission bus bus (usually a substation where one or more
    tranmission lines start/end).

    :param isSwing: flag if the bus is the swing bus
      (sets the reference angle for the system)
    """
    def __init__(self,name=None,index=None,isSwing=False):
        update_attributes(self,locals()) #load in inputs
        self.generators,self.loads=[],[]
        self.init_optimization()

    def angle(self,time): return self.get_variable('angle',time,indexed=True)
    def price(self,time): 
        return dual(self.get_constraint('power balance', time))
    def Pgen(self,t,evaluate=False):
        if evaluate: return sum(value(gen.power(t)) for gen in self.generators)
        else: return sum(gen.power(t) for gen in self.generators)
    def Pload(self,t,evaluate=False):
        if evaluate: return sum(value(ld.power(t)) for ld in self.loads)
        else: return sum(ld.power(t) for ld in self.loads)
    def power_balance(self,t,Bmatrix,allBuses):
        if len(allBuses)==1: lineFlowsFromBus=0
        else: lineFlowsFromBus=sum([Bmatrix[self.index][otherBus.index]*otherBus.angle(t) for otherBus in allBuses]) #P_{ij}=sum_{i} B_{ij}*theta_j ???
        return sum([ -lineFlowsFromBus,-self.Pload(t),self.Pgen(t) ])
    def create_variables(self,times):
        self.add_children(self.generators,'generators')
        self.add_children(self.loads,'loads')
        logging.debug('added bus {} components - generators and loads'.format(self.name))
#        if len(self.generators)<50:
        for gen in self.generators: gen.create_variables(times)
#        else:
#            for gen in self.generators:
#                threading.Thread(target=_call_generator_create_variables,args=(gen,times)).start()
#            else:
#                for th in threading.enumerate():
#                    if th is threading.current_thread(): continue
#                    else: th.join()

        logging.debug('created generator variables')
        for load in self.loads: load.create_variables(times)
        logging.debug('created load variables')
        self.add_variable('angle',index=times.set)
        logging.debug('created bus variables ... returning')
        return
    def create_objective(self,times): return self.cost_first_stage(times) + self.cost_second_stage(times)
    def cost_first_stage(self,times):
        return sum(gen.cost_first_stage(times) for gen in self.generators) + \
            sum(load.cost_first_stage(times) for load in self.loads)
    def cost_second_stage(self,times):
        return sum(gen.cost_second_stage(times) for gen in self.generators) + \
            sum(load.cost_second_stage(times) for load in self.loads)
    def create_constraints(self, times, Bmatrix, buses, include_children=True):
        if include_children:
            for gen in self.generators: gen.create_constraints(times)
            for load in self.loads: load.create_constraints(times)
        nBus=len(buses)
        for time in times:
            self.add_constraint('power balance',time, self.power_balance(time,Bmatrix,buses)==0) #power balance must be zero
            if nBus>1 and self.isSwing:
                self.add_constraint('swing bus',time, self.angle(time)==0)#swing bus has angle=0
        return
    # def clear_constraints(self):
    #     self.constraints={}
    #     for gen in self.generators: gen.clear_constraints()
    #     for load in self.loads: load.clear_constraints()

    def iden(self,t):   return str(self)+str(t)
    def __str__(self):  return 'i{ind}'.format(ind=self.index)

class PowerSystem(OptimizationProblem):
    '''
    Power systems object which is the container for all other components.

    :param generators: list of :class:`~powersystem.Generator` objects
    :param loads: list of :class:`~powersystem.Load` objects
    :param lines: list of :class:`~powersystem.Line` objects

    Other settings are inherited from `user_config`.
    '''
    def __init__(self, generators, loads, lines=None):
        # load in inputs
        update_attributes(self, locals(),
            exclude=['generators', 'loads', 'lines']) 
        self.reserve_fixed = user_config.reserve_fixed
        self.reserve_load_fraction = user_config.reserve_load_fraction

        if lines is None: lines=[]

        buses = self.make_buses_list(loads, generators)
        self.create_admittance_matrix(buses, lines)
        self.init_optimization()

        self.add_children(buses, 'buses')
        self.add_children(lines, 'lines')

        self.is_stochastic = len(filter(lambda gen: gen.is_stochastic, generators))>0
        self.shedding_mode = False

    def make_buses_list(self, loads, generators):
        """
        Create list of :class:`powersystems.Bus` objects
        from the load and generator bus names. Otherwise
        (as in ED,UC) create just one (system)
        :class:`powersystems.Bus` instance.

        :param loads: a list of :class:`powersystems.Load` objects
        :param generators: a list of :class:`powersystems.Generator` objects
        :returns: a list of :class:`powersystems.Bus` objects
        """
        busNameL=[]
        busNameL.extend(getattrL(generators,'bus'))
        busNameL.extend(getattrL(loads,'bus'))
        busNameL = pd.Series(pd.unique(busNameL)).dropna().tolist()

        if len(busNameL) == 0:
            busNameL = [None]
            
        buses = []
        swingHasBeenSet = False

        for b, busNm in enumerate(busNameL):
            newBus = Bus(name=busNm, index=b)
            for gen in generators:
                if gen.bus == newBus.name: 
                    newBus.generators.append(gen)
                if not swingHasBeenSet: 
                    newBus.isSwing = swingHasBeenSet = True
            for ld in loads:
                if ld.bus == newBus.name:
                    newBus.loads.append(ld)
            buses.append(newBus)
        return buses
    def create_admittance_matrix(self, buses, lines):
        """
        Creates the admittance matrix (B),
        with elements = total admittance of line from bus i to j.
        Used in calculating the power balance for OPF problems.

        :param buses: list of :class:`~powersystems.Line` objects
        :param lines: list of :class:`~powersystems.Bus` objects
        """
        nB=len(buses)
        self.Bmatrix=np.zeros((nB,nB))
        namesL=[bus.name for bus in buses]
        for line in lines:
            busFrom = buses[namesL.index(line.frombus)]
            busTo = buses[namesL.index(line.tobus)]
            self.Bmatrix[busFrom.index,busTo.index] += -1/line.reactance
            self.Bmatrix[busTo.index,busFrom.index] += -1/line.reactance
        for i in range(0,nB):
            self.Bmatrix[i,i]=-1*sum(self.Bmatrix[i,:])
    def loads(self): return flatten(bus.loads for bus in self.buses)
    def generators(self): return flatten(bus.generators for bus in self.buses)
    def create_variables(self,times):
        self.add_variable('cost_first_stage')
        self.add_variable('cost_second_stage')
        self.add_set('times', times._set, ordered=True)
        times.set=self._model.times
        for bus in self.buses:  bus.create_variables(times)
        for line in self.lines: line.create_variables(times)
        logging.debug('... created power system vars... returning')

    def cost_first_stage(self,scenario=None): return self.get_component('cost_first_stage',scenario=scenario)
    def cost_second_stage(self,scenario=None): return self.get_component('cost_second_stage',scenario=scenario)
    def create_objective(self,times):
        self.add_objective(self.cost_first_stage()+self.cost_second_stage())
    def create_constraints(self, times, include_children=True):
        if include_children:
            for bus in self.buses: 
                bus.create_constraints(times, self.Bmatrix, self.buses)
            for line in self.lines: 
                line.create_constraints(times, self.buses)

        # system reserve constraint
        self._has_reserve = not self.shedding_mode and \
            (self.reserve_fixed>0 or self.reserve_load_fraction>0)
        if self._has_reserve:
            for time in times:
                required_generation_availability = self.reserve_fixed + (1.0 + self.reserve_load_fraction) * sum(load.power(time) for load in self.loads())
                generation_availability = sum(gen.power_available(time) for gen in self.generators())
                self.add_constraint('reserve', generation_availability >= required_generation_availability, time=time )

        self.add_constraint('system_cost_first_stage',self.cost_first_stage()==sum(bus.cost_first_stage(times) for bus in self.buses))
        self.add_constraint('system_cost_second_stage',self.cost_second_stage()==sum(bus.cost_second_stage(times) for bus in self.buses))
    def iden(self,time=None):
        name='system'
        if time is not None: name+='_'+str(time)
        return name

    
    def total_scheduled_load(self):
        return sum([load.schedule for load in self.loads()]) 
    
    def total_scheduled_generation(self):
        return sum(gen.schedule for gen in self.generators() if not gen.is_controllable)    

    def get_generators_noncontrollable(self):
        return filter(lambda gen: not gen.is_controllable, self.generators())
        
    def get_generators_without_scenarios(self):
        return filter(lambda gen: getattr(gen,'is_stochastic',False)==False, self.generators())

    def get_generator_with_scenarios(self):
        gens = filter(lambda gen: getattr(gen,'is_stochastic',False), self.generators())
        if len(gens)>1: raise NotImplementedError('Dont handle the case of multiple stochastic generators')
        elif len(gens)==0: return []
        else: return gens[0]
    def get_generator_with_observed(self):
        return filter(lambda gen: getattr(gen,'observed_values',None) is not None, self.generators())[0]

    def get_finalconditions(self, sln):
        times = sln.times

        tEnd = times.last_non_overlap() # like 2011-01-01 23:00:00
        tEndstr = times.non_overlap().last() # like t99

        status = sln.generators_status

        for gen in self.generators():
            g = str(gen)
            stat = status[g]
            if sln.is_stochastic:
                gen.finalstatus = dict(
                    power=sln.generators_power[g][tEnd],
                    status=sln.generators_status[g][tEnd],
                    hoursinstatus=gen.gethrsinstatus(times.non_overlap(), stat)
                    )
            else:
                gen.finalstatus = gen.getstatus(tEndstr, 
                    times.non_overlap(), stat)
        return

    def set_initialconditions(self, initTime):
        for gen in self.generators():
            finalstatus = getattr(gen, 'finalstatus', {})
            if finalstatus:
                gen.set_initial_condition(time=initTime, **finalstatus)
                del gen.finalstatus
        return


    def solve_problem(self, times):
        try: 
            instance = self.solve()
        except OptimizationError:
            #re-do stage, with load shedding allowed
            logging.critical('stage infeasible, re-run with load shedding.')
            self.allow_shedding(times)
            try:
                instance = self.solve()
            except OptimizationError:
                scheduled, committed = self.debug_infeasibe(times)
                raise OptimizationError('failed to solve withs shedding.')
        return instance

    def resolve_stochastic_with_observed(self, instance, sln):
        s = sln.scenarios[0]        
        self._model = instance.active_components(pyomo.Block)[s]
        self.is_stochastic = False
        self.stochastic_formulation = False
        
        self._resolve_problem(sln)
        
        # re-store the generator outputs and costs
        sln._resolved = True
        sln._get_outputs(resolve=True)
        sln._get_costs(resolve=True)

        self.is_stochastic = True
        self.disallow_shedding()
        return

    def resolve_determinisitc_with_observed(self, sln):
        # store the useful expected value solution information
        sln.expected_status = sln.generators_status.copy()
        sln.expected_power = sln.generators_power.copy()
        sln.expected_fuelcost = sln.fuelcost.copy()
        sln.expected_totalcost = sln.totalcost_generation.copy()
        sln.expected_load_shed = float(sln.load_shed)

        # resolve the problem
        self._resolve_problem(sln)

        # re-calc the generator outputs and costs
        sln._resolved = True
        sln._get_outputs()
        sln._get_costs()

        sln.observed_fuelcost = sln.fuelcost
        sln.observed_totalcost = sln.totalcost_generation
        self.disallow_shedding()
        return

    def _set_load_shedding(self, to_mode):
        '''set system mode for load shedding'''
        for load in filter(lambda ld:
            ld.sheddingallowed, self.loads()):
            load.shedding_mode = to_mode

    def _set_gen_shedding(self, to_mode):
        for gen in filter(lambda g: 
            not g.is_controllable and g.sheddingallowed, self.generators()):
            gen.shedding_mode = to_mode

    
    def allow_shedding(self, times, resolve=False):
        self.shedding_mode = True
        self._set_load_shedding(True)
        self._set_gen_shedding(True)
        
        const_times = times.non_overlap() if resolve else times
        
        # make load power into a variable instead of a param
        for load in self.loads():
            load.create_variables(times)  # need all for the .set attrib
            load.create_constraints(const_times)
        
        for gen in filter(lambda g: 
            getattr(g, 'shedding_mode', False), self.generators()):
            # create only the power_used var, don't reset the power param
            gen.create_variables_shedding(times) 
            gen.create_constraints(const_times)
        
        # recalc the power balance constraint
        for bus in self.buses:
            for time in times:
                bus._remove_component('power balance', time)
            bus.create_constraints(const_times, 
                self.Bmatrix, self.buses, include_children=False)
        
        # reset objective
        self._model.objective = None
        self.create_objective(const_times)
        # re-create system cost constraints
        self.create_constraints(const_times, include_children=False)
        # recreating all constraints would be simpler
        # but would take a bit longer

    def disallow_shedding(self):
        # change shedding allowed flags for the next stage
        self.shedding_mode = False
        self._set_load_shedding(False)
        self._set_gen_shedding(False)


    def _resolve_problem(self, sln):
        times = sln.times_non_overlap

        # reset the constraints
        self._remove_all_constraints()
        # dont create reserve constraints
        self.reserve_fixed = 0
        self.reserve_load_fraction = 0
        # recreate constraints only for the non-overlap times
        self.create_constraints(times)
        # reset objective to only the non-overlap times
        self._model.objective = None
        self.create_objective(times)
        
        # set wind to observed power
        gen = self.get_generator_with_observed()
        gen.set_power_to_observed(times)

        # fix statuses for all units
        self.fix_binary_variables()
        
        # store original problem solve time
        self.full_sln_time = self.solution_time
        full_mipgap = self.mipgap
        
        logging.info('resolving with observed values')
        try: self.solve()
        except OptimizationError:
            if user_config.faststart_resolve:
                self._resolve_with_faststarts(sln)
            else:
                # just shed the un-meetable load and calculate cost later
                self.allow_shedding(sln.times, resolve=True)
                try: self.solve()
                except OptimizationError:
                    scheduled, committed = self.debug_infeasibe(
                        sln.times, resolve_sln=sln)
                    raise
        
        self.resolve_solution_time = float(self.solution_time)
        self.solution_time = float(self.full_sln_time)
        
        if self.mipgap: self.mipgap_resolve = float(self.mipgap)
        if full_mipgap: self.mipgap = float(full_mipgap)
        
        logging.info('resolved instance with observed values (in {}s)'.format(
            self.resolve_solution_time))

    def _resolve_with_faststarts(self, sln):
        '''allow faststart units to be started up to meet the load'''
        self._unfix_variables()
        self._fix_non_faststarts(sln.times)
        logging.warning('allowing fast-starting units')
        try: self.solve()
        except OptimizationError:
            self._unfix_variables()
            self._fix_non_faststarts(sln.times, fix_power=False)
            logging.warning(
                'allowing non fast-starters to change power output')
            try: self.solve()
            except OptimizationError:
                logging.warning('allowing load shedding')
                self.allow_shedding(sln.times, resolve=True)
                try: self.solve()
                except OptimizationError:
                    scheduled, committed = self.debug_infeasibe(
                        sln.times, resolve_sln=sln)
                    raise
                
                
    def _fix_non_faststarts(self, times, fix_power=True):
        '''
        fix non-faststart units - both power and status
        (unless this is infeasible, then only fix status)
        the idea is that fast-starts should be contributing power
        only for system security, not economics
        '''
        for gen in filter(lambda gen: \
            (not gen.faststart) and gen.is_controllable, self.generators()):
            for time in times:
                gen.status(time).fixed = True
                if fix_power: gen.power(time).fixed = True


    def debug_infeasibe(self, times, resolve_sln=None):
        generators = self.generators()
        if resolve_sln: 
            windgen = self.get_generator_with_observed()
            
            scheduled = pd.DataFrame({
                'expected_power': resolve_sln.generators_power.sum(axis=1).values,
                'expected_wind': windgen.schedule.ix[times.non_overlap()],
                'observed_wind': windgen.observed_values.ix[times.non_overlap()],
                })
            scheduled['net_required'] = \
                scheduled.expected_wind - scheduled.observed_wind
        else:                
            scheduled = pd.DataFrame({
                'load': self.total_scheduled_load().ix[times.strings.values], 
                'generation': self.total_scheduled_generation().ix[times.strings.values]})
            scheduled['net_required'] = scheduled['load'] - scheduled.generation        
        print 'total scheduled\n', scheduled
        
        if resolve_sln:
            committed = pd.DataFrame(dict(
                Pmin=[gen.pmin for gen in generators],
                Pmax=[gen.pmax for gen in generators],
                rampratemin=[getattr(gen, 'rampratemin', None) for gen in generators],
                rampratemax=[getattr(gen, 'rampratemax', None) for gen in generators],
                )).T
            print 'generator limits\n', committed
        else:
            gens = filter(lambda gen: \
                gen.is_controllable and gen.initial_status == 1,
                self.generators())
            committed = pd.Series(dict(
                Pmin=sum(gen.pmin for gen in gens),
                Pmax=sum(gen.pmax for gen in gens),
                rampratemin=pd.Series(gen.rampratemin for gen in gens).sum(),
                rampratemax=pd.Series(gen.rampratemax for gen in gens).sum(),
                ))
            print 'total committed\n', committed

        if resolve_sln: 
            print 'expected status\n', resolve_sln.generators_status
            ep = resolve_sln.generators_power
            ep['net_required'] = scheduled.net_required.values
            print 'expected power\n', ep
        else:
            print 'initial_status\n'
            print pd.Series([gen.initial_status for gen in self.generators()])
        return scheduled, committed



"""
A module to workaround a memory leak in coopr by saving the results 
of each day of a rolling unit commitment to disk (in HDF format)
and reloading them to run the next day as a memory independent subprocess.
"""

import os, logging, sys
import pandas as pd
from pandas import Series, DataFrame
from commonscripts import gen_time_dataframe, set_trace
from config import user_config

from schedule import TimeIndex
from get_data import parse_standalone
import pkg_resources

def wipe_storage():
    try: 
        os.remove(user_config.store_filename)
        print 'wipe storage'
    except OSError: pass
    

def get_storage():
    return pd.HDFStore(user_config.store_filename)

        
def store_times(tstage, storage=None):
    if storage is None:
        storage = get_storage()
    storage['times'] = tstage.strings
    return storage

def init_store(power_system, times, data):
    '''create the store before the first stage'''
    wipe_storage()
    storage = get_storage()
    generators = power_system.generators()
    
    # store the problem info read from the spreadsheets
    for key, df in data.iteritems():
        key = 'data_' + key
        storage[key] = df
    
    stages = len(times) #stage_times
    t = [times[0].initialTime]
    
    # store first stage initial state
    storage['power'] = gen_time_dataframe(generators, t, 
        values = [[gen.initial_power for gen in generators]])
    storage['status'] = gen_time_dataframe(generators, t, 
        values = [[gen.initial_status for gen in generators]])
    storage['hrsinstatus'] = gen_time_dataframe(generators, t, 
        values = [[gen.initial_status_hours for gen in generators]])

    # setup empty containers for variables
    storage['load_shed'] = Series()
    storage['gen_shed'] = Series()
    storage['expected_cost'] = DataFrame()
    storage['observed_cost'] = DataFrame()
    storage['expected_status'] = DataFrame()
    storage['expected_power'] = DataFrame()
    storage['expected_fuelcost'] = DataFrame()
    storage['observed_fuelcost'] = DataFrame()
    
    # setup one-per-stage results
    storage['solve_time'] = Series(index=range(stages))
    storage['mipgap'] = Series(index=range(stages))
    
    # store configuration
    storage['configuration'] = Series(user_config)
    
    
    storage['version'] = Series({
        'minpower': pkg_resources.get_distribution('minpower').version,
        # try storing version number of the current working directory 
        'problem': _get_problem_version(),
        })


    return storage

def _get_problem_version():
    version = None
    possible_init = os.path.join(
        os.path.abspath(user_config.directory), '__init__.py')
    if os.path.exists(possible_init):
        vstr = open(possible_init, 'r').read()
        version = vstr.split('"')[1].rstrip('"')
    return version
    
def store_state(power_system, times, sln=None):
    storage = get_storage()
    generators = power_system.generators()
   
    stg = sln.stage_number
    table_append(storage, 'power', sln.generators_power)
    table_append(storage, 'status', sln.generators_status)
    table_append(storage, 'load_shed', sln.load_shed_timeseries)
    table_append(storage, 'gen_shed', sln.gen_shed_timeseries)
    
    tEnd = times.last_non_overlap()
    storage['hrsinstatus'] = gen_time_dataframe(generators, [tEnd], 
        values = [
            [gen.finalstatus['hoursinstatus'] for gen in generators]
        ])
    
    _add_tbl_val(storage, 'solve_time', stg, sln.solve_time)
    _add_tbl_val(storage, 'mipgap', stg, sln.mipgap)
    
    if sln._resolved:
        table_append(storage, 'observed_cost', sln.observed_totalcost)
        table_append(storage, 'observed_fuelcost', sln.observed_fuelcost)
        table_append(storage, 'expected_cost', sln.expected_totalcost)
        table_append(storage, 'expected_fuelcost', sln.expected_fuelcost)
        table_append(storage, 'expected_power', sln.expected_power)
        table_append(storage, 'expected_status', sln.expected_status)
    else:
        table_append(storage, 'expected_cost', sln.totalcost_generation)
        table_append(storage, 'expected_fuelcost', sln.fuelcost)
    return storage
    
def load_state():
    storage = get_storage()
    user_config.update(storage['configuration'].to_dict())
    
    # set up the times
    startidx = int(storage['times'][0].strip('t'))
    times = TimeIndex(storage['times'].index, startidx)
    intervalhrs = (times.strings.index[1] - times.strings.index[0]).total_seconds() / 3600.0
    times._int_division = user_config.hours_commitment / intervalhrs
    times._int_overlap = user_config.hours_overlap / intervalhrs
    if len(times) <= times._int_division:
        # dont set overlap for last stage
        times._int_overlap = 0

    # create power_system
    power_system, times, scenario_tree = parse_standalone(storage, times)
    generators = power_system.generators()
    
    # set up initial state
    t = times.initialTime
    
    
    for gen in generators: 
        g = str(gen)
        gen.set_initial_condition(t, 
            power=storage['power'][g][t],
            status=storage['status'][g][t],
            hoursinstatus=storage['hrsinstatus'][g][t])
    
    return power_system, times, scenario_tree

def repack_storage():
    '''compress that ballooning storage'''
    # http://stackoverflow.com/questions/13089359/mystery-when-storing-a-dataframe-containing-strings-in-hdf-with-pandas
    os.system('ptrepack {f} {f}.copy; mv {f}.copy {f};'.format(
        f=user_config.store_filename))
    
def _add_tbl_val(storage, tablename, index, value):
    tbl = storage[tablename]
    tbl[index] = value
    storage[tablename] = tbl
    
def table_append(store, name, newvals):
    store[name] = store[name].append(newvals)
    return
    
try: 
    import tables as tb
    import atexit
    
    # FIXME - not working
    # make pytables quiet down
    # http://www.pytables.org/moin/UserDocuments/AtexitHooks
    def quiet_table_close(verbose=False):
        open_files = tb.file._open_files
        are_open_files = len(open_files) > 0
        if verbose and are_open_files:
            print >> sys.stderr, "Closing remaining open files:",
        for fileh in open_files.keys():
            if verbose:
                print >> sys.stderr, "%s..." % (open_files[fileh].filename,),
            open_files[fileh].close()
            if verbose:
                print >> sys.stderr, "done",
        if verbose and are_open_files:
            print >> sys.stderr

    atexit.register(quiet_table_close)
    
except ImportError:
    logging.debug('could not load pytables - cannot use standalone mode.')



# minpower's setup script

# make a new release by using 
# seed release --dry-run
# and then 
# seed release

from setuptools import setup, find_packages

version_number = open('minpower/__init__.py').read().split('"')[1].rstrip('"')

setup(
    name = "minpower",
    version = version_number,
    download_url = "https://github.com/adamgreenhall/minpower" + \
        "/zipball/v{v}".format(v=version_number),
    entry_points="""
    [console_scripts]
    minpower = minpower.solve:main
    standalone_minpower = minpower.solve:standaloneUC
    hyak_minpower = minpower.experiments.minpower_hyak:main
    """,

    package_data={
        'minpower.configuration': ['minpower.cfg'],
        'minpower.tests': [
            '*.csv', 
            '*/*.csv',
            '*/*/*.csv']},

    install_requires=[
        'Coopr>=3.2.6148', #3.1.5409
        'coopr.core>=1.0',
        'pyutilib>=4.0',
        'numpy>=1.6.1',
        'pandas>=0.10',
        'python-dateutil>=1.4.1',
        'ordereddict>=1.1',
        'PyYAML>=3.10',
    ],
    tests_require=[
        'nose',
        'coverage',
        'objgraph'
        ],
    
    # it helps to have seed if you are going to make releases
    # but it is not required for setup
    # setup_requires=['seed >= 0.2.12'],

    description = "power systems optimization made beautiful",
    author = "Adam Greenhall",
    author_email = "minpower@adamgreenhall.com",
    url = "http://minpowertoolkit.com/",

    packages = find_packages(),
    keywords = ["power systems","optimization"],

    classifiers = [
        "Development Status :: 4 - Beta",
        "Environment :: Other Environment",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Education",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Mathematics",
        ],
    long_description = """
power systems tools made beautiful
-----------------------------------------

* Solves ED, OPF, and UC problems.
* Problems can be defined in simple spreadsheets.
* Visualizations are created for the answers.
* Many solvers are supported.


* `Full documentation and tutorials <http://minpowertoolkit.com>`_
* `Actively developed <http://github.com/adamgreenhall/minpower>`_

"""
)


"""
Time and schedule related models.
"""

import dateutil
from datetime import timedelta, datetime
from commonscripts import *
from operator import attrgetter

def get_schedule(filename):
    return ts_from_csv(filename)

def make_times_basic(N):
    '''make a :class:`schedule.TimeIndex` of N times with hourly interval'''
    return TimeIndex(date_range('00:00:00', periods=N, freq='H'))
    
def just_one_time():
    '''make a TimeIndex with just one time in it'''
    return make_times_basic(1)

def make_constant_schedule(times, power=0):
    return Series(power, times.strings.values)


class TimeIndex(object):
    '''a list of times (underlying model is pandas.Index)'''
    def __init__(self, index, str_start=0):
        strings = ['t%02d'%(i+str_start) for i in range(len(index))]
        self.times = index.copy()
        self.strings = Series(strings,index=self.times)
        self._set = self.strings.values.tolist()
        
        self.get_interval()
        self.Start = self.times[0]
        self.startdate = self.Start.date()
        
        self.End = self.times[-1] + self.interval
        self.span = self.End-self.Start
        self.spanhrs = hours(self.span)
        
        self.set_initial()
                
        self._int_overlap = 0
        self._int_division = len(self)
        self._str_start = str_start

    def set_initial(self,initialTime=None): 
        if initialTime: self.initialTime= initialTime
        else: 
            self.initialTime = pd.Timestamp(self.Start - self.interval)
            self.initialTime.index = 'Init'
        self.initialTimestr = 'tInit'
        
    def get_interval(self):
        freq = self.strings.index.freq
        if freq is not None:
            self.interval = freq
            if self.interval.freqstr=='H':
                self.intervalhrs = self.interval.n
            else:
                self.intervalhrs = self.interval.nanos / 1.0e9 / 3600.0
        else:            
            self.interval = self.times[1] - self.times[0]
            self.intervalhrs = self.interval.total_seconds() / 3600.0
        return

    def __contains__(self, item): return item in self.times
    def __repr__(self): return repr(self.times)
    def __len__(self): return len(self.times)
    def __getitem__(self, i, circular=False): 
        if i==-1 and not circular:
            return self.initialTime
        else:
            return self.strings[i]
    def last(self): return self.__getitem__(-1, circular=True)
    
    def __getslice__(self, i, j): return self.strings[i:j]

    def non_overlap(self):
        if self._int_overlap > 0:
            return TimeIndex(self.strings.index[:-1-self._int_overlap+1], self._str_start)
        else: 
            return self
        return 
    
    def post_horizon(self):
        if len(self) > self._int_division:
            str_start = int(self.strings.ix[self._int_division+1].strip('t'))            
            return TimeIndex(self.strings.index[self._int_division+1:], str_start)
        else: 
            return Series()
        return 
    
    
    def last_non_overlap(self): return self.strings.index[-1-self._int_overlap]


    def subdivide(self, division_hrs=24, overlap_hrs=0):
        int_division = int(division_hrs / self.intervalhrs)
        int_overlap = int(overlap_hrs / self.intervalhrs)
        subsets = []
        for stg in range(int(len(self) / int_division)):
            start = stg * int_division
            end_point = start + int_division + int_overlap
            end_point = min(end_point, len(self))

            subset = TimeIndex(self.times[start:end_point], start)
            subset._int_overlap = int_overlap
            subset._int_division = int_division
            subsets.append(subset)

        if len(subsets[-1]) <= int_division:
            subsets[-1]._int_overlap = 0
        return subsets
        
def is_init(time):
    return getattr(time,'index',None)=='Init'



"""
Provide the defaults and configuration for other modules.
`user_config` is treated as a global in minpower.
"""
import os
from ConfigParser import SafeConfigParser
from commonscripts import DotDict, joindir

generator_kinds=[
    'generic',
    'coal','nuclear',
    'ngst','ngcc',
    'nggt',
    'wind',
    'chp'
    ]

generator_defaults=dict(
    Pmin=dict(generic=0,wind=0),

    costcurvestring=dict(
        generic=            '3000+25P+.005P^2',
        coal=               '3011+20.14P+.0084P^2',
        nuclear=            '5178+12.85P+.0014P^2',
        steamturbine=       '4793+42.75P+.0024P^2',
        combustionturbine=  '9586+42.75P+.0012P^2',
        wind=               '0'
        ),
    is_controllable=dict(generic=True,wind=False),
    power=dict(generic=None),

    Pmax=dict(
        generic=             500,
        coal=                550,
        nuclear=            1117,
        nggt=                 90,
        ngcc=                420,
        ngst   =             550,#assume equal to coal fired steam
        wind   =             100,
        ),

    minuptime=dict(
        generic=0,
        coal=24,
        nuclear=120,
        nggt=0,
        ngcc=6,
        ngst=24, #assume equal to coal fired steam
        wind=0,
        ),

    mindowntime=dict(
        generic=0,
        coal=2,
        nuclear=18,
        nggt=0,
        ngcc=12,
        ngst=12,#assume equal to coal fired steam
        wind=0,
        ),

    fuelcost=dict(
        generic=    1,
        coal=       2.0,   #from EIA national average coal price: http://www.eia.gov/electricity/monthly/
        ngst=       4.0,   #midrange forecast from NWPP plan: http://www.nwcouncil.org/energy/powerplan/6/
        ngcc=       4.0,
        nggt=       4.0,
        nuclear =   0.65,  #from NWPP plan - Table I-34: Forecast nuclear fuel prices (2006$/MMBtu)
        wind=       0,
        ),


    startupcost=dict(
        generic=              0, #if kind not specified, no startup cost
##from Henry Louie thesis
#        coal=            107800,
#        nuclear=         283500,
#        ngst=             73500,
#        nggt=             36750,
#        wind=                 0,
#        ngcc=             73500, #assume same as NG ST
##from WECC Planning Study, via B.Palmintier - http://www.eia.gov/oiaf/beck_plantcosts/
        coal=     3581,
        nuclear=100000,
        ngst   =  3581,
        nggt   =  7811,
        ngcc   =  9250,
        wind   =     0,
        )
    )


parser = SafeConfigParser()
parser.read([
    # the minpower default set, from the minpower/configuration directory 
    joindir(os.path.split(__file__)[0], 'configuration/minpower.cfg'), 
    # the user's overrides, from the home directory
    os.path.expanduser('~/minpower.cfg'),
    os.path.expanduser('~/.minpowerrc'),
    ],
    # add another override in the specific case directory??
    )


m = 'minpower'
user_config = DotDict(dict(
    duals = parser.getboolean(m, 'duals'),
    breakpoints = parser.getint(m, 'breakpoints'),
    hours_commitment = parser.getint(m, 'hours_commitment'),
    hours_overlap = parser.getint(m, 'hours_overlap'),
    
    cost_load_shedding = parser.getfloat(m, 'cost_load_shedding'),
    cost_wind_shedding = parser.getfloat(m, 'cost_wind_shedding'),
    dispatch_decommit_allowed = \
        parser.getboolean(m, 'dispatch_decommit_allowed'),
    solver = parser.get(m, 'solver'),
    mipgap = parser.getfloat(m, 'mipgap'), 

    reserve_fixed = parser.getfloat(m, 'reserve_fixed'),
    reserve_load_fraction = \
        parser.getfloat(m, 'reserve_load_fraction'),

    faststart_resolve = parser.getboolean(m, 'faststart_resolve'),

    visualization = parser.getboolean(m, 'visualization'),
    logging_level = parser.getint(m, 'logging_level'),
    problem_file = parser.getboolean(m, 'problem_file'),
    output_prefix = parser.getboolean(m, 'output_prefix'),
    debugger = parser.getboolean(m, 'debugger'),


    scenarios = parser.getint(m, 'scenarios'),
    deterministic_solve = parser.getboolean(m, 'deterministic_solve'),
    perfect_solve = parser.getboolean(m, 'perfect_solve'),
    scenarios_directory = parser.get(m, 'scenarios_directory'),
    
    standalone = parser.getboolean(m, 'standalone'),

    wind_forecast_adder = parser.getfloat(m, 'wind_forecast_adder'),
    wind_multiplier = parser.getfloat(m, 'wind_multiplier'),
    load_multiplier = parser.getfloat(m, 'load_multiplier'),

    pmin_multiplier = parser.getfloat(m, 'pmin_multiplier'),
    ramp_limit_multiplier = parser.getfloat(m, 'ramp_limit_multiplier'),

    ignore_minhours_constraints = parser.getboolean(
        m, 'ignore_minhours_constraints'),
    ignore_ramping_constraints = parser.getboolean(
        m, 'ignore_ramping_constraints'),
    ignore_pmin_constraints = parser.getboolean(
        m, 'ignore_pmin_constraints'),
    
    on_complete_script = parser.get(m, 'on_complete_script'),
    
    # HACKs to help out resetting the config in testing
    directory = parser.get(m, 'directory'),
    store_filename = parser.get(m, 'store_filename'),
    ))

available_solvers = []
for solver in parser.options('available_solvers'):
  if parser.getboolean('available_solvers', solver):
      available_solvers.append(solver)



"""
Short, commonly used scripts. 
Many of them are one liners.
"""

import os
import sys
import csv
import logging
import itertools
import operator
import datetime
import numpy as np
import pandas as pd
from pandas import DataFrame, Series, date_range

from pprint import pprint
try: # for development
    from ipdb import set_trace #pudb
    import IPython
    from IPython.frontend.terminal.embed import InteractiveShellEmbed
#    from IPython.config.loader import Config
#    
#    cfg = Config()
#    cfg.InteractiveShellEmbed.prompt_in1="myprompt [\\#]> "
#    cfg.InteractiveShellEmbed.prompt_out="myprompt [\\#]: "
#    cfg.InteractiveShellEmbed.profile=ipythonprofile
    ipython_shell = InteractiveShellEmbed() #config=cfg, user_ns=namespace, banner2=banner)
except: 
    from pdb import set_trace

class StreamToLogger(object):
   """
   Fake file-like stream object that redirects writes to a logger instance.
   """
   def __init__(self, logger=logging, log_level=logging.INFO):
      self.logger = logger
      self.log_level = log_level
      self.linebuf = ''
 
   def write(self, buf):
      for line in buf.rstrip().splitlines():
         self.logger.log(self.log_level, line.rstrip())


def gen_time_dataframe(generators, times, values=()):
    kwargs = dict(columns = [str(g) for g in generators])
    try: kwargs['index'] = times.strings.index
    except AttributeError: kwargs['index'] = times
        
    if values:
        values = np.array(values)
        if values.shape != (len(times),len(generators)):
            values = values.T
        df = DataFrame(values, **kwargs)
    else: 
        df = DataFrame(**kwargs)
        
    df.index.name = 'time'
    return df
    
def ts_from_csv(filename,index_col=0, squeeze=True, timezone=None, is_df=True, **kwargs):
    kwargs['header']=0 if is_df else None
    
    df = pd.read_csv(filename, index_col=index_col, squeeze=squeeze, **kwargs)
    df.index = pd.DatetimeIndex(df.index)
    if timezone is not None: 
        # pandas seems to convert any stamps to UTC in the DatetimeIndex call
        df.index = df.index.tz_localize('UTC').tz_convert(timezone)
    return df

def bool_to_int(x): return 1 if x else 0

class DotDict(dict):
    '''a dict with dot notation access'''
    def __getattr__(self, attr):
        return self.get(attr)
    def copy(self): return DotDict(dict(self))
    
    __setattr__= dict.__setitem__
    __delattr__= dict.__delitem__

###### matrix stuff #######
def elementwiseAdd(La,Lb): return map(operator.add, La,Lb)
def transpose(listoflists): return map(None,*listoflists)
def flatten(listoflists):
    '''Flatten one level of nesting'''
    return list(itertools.chain.from_iterable(listoflists))
def within(x, val=0, eps=1e-3): return (val-eps) <= x <= (val+eps)


def replace_all(seq, obj, replacement):
    def with_index(seq):
        for i in xrange(len(seq)): yield i, seq[i]

    for i, elem in with_index(seq):
        if elem == obj: seq[i] = replacement
    return seq
    
def pairwise(iterable):
    "s -> (s0,s1), (s1,s2), (s2, s3), ..."
    a, b = itertools.tee(iterable)
    next(b, None)
    return itertools.izip(a, b)

##### csv stuff #####
def readCSV(filenm, validFields='all'):
    """
    Read data from a csv into a list of lists. 
    Does parsing of each cell with :func:`csvDataConvert`.
    Field order doesn't matter. If validFields is not 'all' and
    a field is not in validFields, its data will be excluded from 
    the output data.
        
    :param filenm: name of the csv file to read
    :param validFields: list of valid field names (case and space insensitive)
    
    :returns: data, a list of lists
    :returns: fieldsChecked, a list of fields 
    """
    csvfile = open(filenm)
    try: dialect = csv.Sniffer().sniff(csvfile.read(4048))
    except: 
        #this generally happens when the file is very short
        #assume dialect is excel
        dialect = csv.excel
    csvfile.seek(0)
    reader = csv.reader(csvfile, dialect)
    def csvDataConvert(inData, repForBlankStrings=None, checkForNumbers=True):
        outData=[]
        for row in inData:
            newRow=[]
            for entry in row:
                entry=entry.strip()
                if entry.strip()=='': newRow.append( repForBlankStrings )
                elif checkForNumbers: newRow.append( convert_str2num(entry) )
                else: newRow.append(entry)
            #if not all(e=='' for e in newRow): 
            outData.append(newRow)
        return outData
    
    data=csvDataConvert(zip(*reader))
    csvfile.close()
    
    try: data=transpose(data) #return data in r,c index order 
    except TypeError:
        print 'data for {file} is length: {n}'.format(file=filenm,n=len(data))
        print 'file is {n}B'.format(n=os.path.getsize(filenm))
        raise
    fields=data.pop(0) #fields are the first row of csv
    
    if validFields=='all': return data,fields
    else: #ensure that each field is in validFields
        
        fieldsChecked=list(fields[:])
        for i,field in enumerate(fields): 
            if field in validFields: continue
            else: 
                fieldSimple=drop_case_spaces(field)
                if fieldSimple in validFields: fieldsChecked[fieldsChecked.index(field)]=fieldSimple #compare without considering case or spaces
                elif fieldSimple is not None: print 'Ignored "{f}" as not a valid input field for {nm}. '.format(f=field,nm=filenm)
        
        return data,fieldsChecked    

def csv2dicts(filename,field_map=None):
    def valid_cell(name,val): return name is not None and val.strip()!='' 
    with open(filename,'r') as csvfile:
        try: dialect = csv.Sniffer().sniff(csvfile.read(4048))
        except: dialect = csv.excel
        csvfile.seek(0)
        raw_data=list(csv.DictReader(csvfile, dialect=dialect))
    if field_map is not None: 
        try: data=[{field_map[drop_case_spaces(name)]:convert_str2num(val) for name,val in row.items() if valid_cell(name,val)} for row in raw_data]
        except: 
            print raw_data
            raise
    else:
        data=[{name:convert_str2num(val) for name,val in row.items() if valid_cell(name,val)} for row in raw_data]
    return data 

def writeCSV(fields,data,filename):
    with open(filename, 'w+') as f:
        writer = csv.writer(f)
        writer.writerow(fields)
        writer.writerows(data)
        
#################### string stuff ##################
def convert_str2num(s):
    s=s.strip()
    try: return int(s)
    except ValueError: 
        try: return float(s)
        except ValueError: return s
def indexCaseSpaceInsensitive(L,s): return map(drop_case_spaces,L).index( drop_case_spaces(s) )
def drop_case_spaces(s): 
    '''get rid of spaces in a string and make lower case. will also work with list of strings'''
    try: return s.lower().replace(' ','').replace('_','')
    except AttributeError: 
        if s is None: return None
        elif isinstance(s, list): return map(drop_case_spaces,s)

def to_percent(val, digits=0): return '{p:.{d}%}'.format(p=val,d=digits)


##################### file stuff ###########################
def splitFilename(fullPathFilenm):
    '''split a filename into its directory, filename, and extension'''
    (dirNm,fullFilenm)=os.path.split(fullPathFilenm)
    (fileNm,extNm)=os.path.splitext(fullFilenm)
    return dirNm, fileNm, extNm
def joindir(dir,file): return os.path.join(dir, file)
            
################### time stuff ###########################
def hours(t): 
    try:  return t.days*24.0 + t.seconds/3600.0 #t is a datetime object
    except AttributeError: return datetime.timedelta(hours=t) #t is a number

####################### class stuff #######################
def getattrL(L,attribute='name'):
    '''get the attribute of each class instance in a list'''
    return [getattr(item,attribute) for item in L]

def update_attributes(instance, variables, exclude=['self'],include=None):
    """Update instance attributes
 
    For example, update(self, locals())
    
    instance: Instance to update via setattr()
    variables: Dictionary of variables
    exclude: Variables to explicitly exclude, defaults to ['self'].
    include: Variables to explicitly include. If set, include overides exclude.
    """
    if include is not None: [setattr(instance,k,v) for k,v in variables.items() if k in include]
    else:
        if 'self' not in exclude: exclude.append('self')
        [setattr(instance, k, v) for k, v in variables.items() if k not in exclude]
